import Settings from "../../settings"
import Action from "../../action"
import UrlUtils from "../../utils/url"
import TypeUtils from "../../utils/type"

export default {
  data: function () {
    return {
      genericIsBusy: false
    }
  },
  computed: {
    _pageBody: function() {
      return document.getElementById("page_body")
    },
  },
  mounted: function() {
    this._executeIfReady(this.$ready, false)
  },
  updated: function() {
    this._executeIfReady(this.$ready, true)
  },
  methods: {
    genericStyles: function (spec) {
      const properties = spec || this.spec
      const styles = {
        'color': properties.color,
        'background-color': properties.backgroundColor
      }

      const padding = properties.padding
      if (TypeUtils.isObject(padding)) {
        TypeUtils.ifNumber(padding.top, top => styles['padding-top'] = `${top}px`)
        TypeUtils.ifNumber(padding.bottom, bottom => styles['padding-bottom'] = `${bottom}px`)
        TypeUtils.ifNumber(padding.left, left => styles['padding-left'] = `${left}px`)
        TypeUtils.ifNumber(padding.right, right => styles['padding-right'] = `${right}px`)
      }

      TypeUtils.ifString(this.$length(properties.width), val => styles['width'] = val)
      TypeUtils.ifString(this.$length(properties.height), val => styles['height'] = val)

      TypeUtils.ifObject(properties.onClick, val => styles['cursor'] = 'pointer')

      return styles
    },
    genericClasses: function (spec) {
      const properties = spec || this.spec
      return properties.styleClasses
    },
    genericHref: function (spec) {
      // if (Settings.reactive) {
      //   return null
      // }

      const properties = spec || this.spec
      const onClick = properties.onClick
      if (TypeUtils.isObject(onClick)) {
        switch (onClick.action) {
          case "windows/open-v1":
          case "windows/openWeb-v1":
            return UrlUtils.htmlUrl(onClick.url)
        }
      }
      return null
    },
    genericOnClick: function(e, spec) {
      const properties = spec || this.spec

      // if (!Settings.reactive) {
      //   event.preventDefault()
      //   event.stopPropagation()
      //   Action.execute(properties.onClick, event.target, this)
      //   return
      // }

      if (!event.metaKey && !event.ctrlKey && event.button === 0) {
        event.preventDefault()
        event.stopPropagation()
        Action.execute(properties.onClick, event.target, this)
      }
    },
    $rel: function(spec) {
      const properties = spec || this.spec
      const onClick = properties.onClick
      if (TypeUtils.isObject(onClick)) {
        if (onClick.action === "windows/openWeb-v1") {
          return "nofollow"
        }
      }
      return null
    },
    $length: function(length) {
      if (TypeUtils.isString(length) || TypeUtils.isNumber(length)) {
        switch(length) {
          case 'wrapContent':
            return null
          case 'matchParent':
            return "100%"
          default:
            return `${length}px`
        }
      }
      return null
    },
    $addViewportChangeListeners: function(handler) {
      window.addEventListener('load', handler)
      window.addEventListener('resize', handler)
      this._pageBody.addEventListener('scroll', handler)
    },
    $removeViewportChangeListeners: function(handler) {
      window.removeEventListener('load', handler)
      window.removeEventListener('resize', handler)
      this._pageBody.removeEventListener('scroll', handler)
    },
    // To be used in `mounted()` and `updated()`
    _executeIfReady: function(handler, updated) {
      if (updated) {
        if (this._mountedUrl !== window.location.href) {
          this._mountedUrl = window.location.href
          handler()
        }
      }
      else {
        this._mountedUrl = window.location.href
        handler()
      }
    },
    $ready: function() {
      // To be overridden
    },
  }
}
