import Url from "./url"
import Storage from "./storage"
import Type from "./type"
import Keys from "../keys"
import Launch from "../utils/launch"

let loading = false

export default class {
  static execute(properties, methodName, controller, jsonHandler) {
    this.showIndicator()
    controller.genericIsBusy = true

    // `fetch()` only supports uppercase
    const method = methodName.toUpperCase()
    const url = Url.jsonUrl(properties['url'])
    console.log(`${method} ${url}`)

    var formData = new FormData()

    if (properties['formDataEncoded']) {
      formData = properties['formData']
    }
    else {
      var params = properties['formData'] || {}
      for (const key in params) {
        formData.append(key, params[key])
      }
      if (!Type.isString(params["authenticity_token"])) {
        formData.append("authenticity_token", Storage.getLocal(Keys.Db.csrfToken))
      }
    }

    let response = null
    const vm = this
    fetch(url, {
      method: method,
      body: method === 'GET' ? null : formData,
      headers: { 'Accept': 'text/html' },
      // Make sure to pass cookies for same origin URLs.
      // Needed for some browsers, e.g. Edge and Android's native.
      credentials: 'same-origin'
    })
    .then(res => {
      vm.hideIndicator()
      controller.genericIsBusy = false

      if (res.status >= 500) {
        Launch.snackbar({ message: 'Server error' })
      }
      else {
        response = res
        return res.json()
      }
    })
    .then(data => {
      console.log('Success', data)
      jsonHandler(data, response)
    })
    .catch(error => console.error('Error:', error))
  }
  
  static showIndicator() {
    loading = true
    setTimeout(() => {
      if (loading) {
        loading = false
        window.vueApp.indicator = true
      }
    }, 200)
  }

  static hideIndicator() {
    loading = false
    window.vueApp.indicator = false
  }
}
